#version 330 core

in vec4 vFinalVertexColor;
in vec3 vWorldNormal;
in vec2 vUVs;
//in vec3 vTangent;

//varyings
in vec3 varNormal;
in vec3 varVertexPosition;
in vec3 varTangent;

uniform vec4 uColor;
uniform sampler2D colorTexture;
uniform sampler2D normalMap;
uniform sampler2D lightMap;
uniform vec2 movement;
uniform int transparent;
uniform float alphaUni;

struct SimpleDirectionalLight
{
	vec3 vColor;
	vec3 vDirection;
	float fAmbientIntensity;
};

struct SpotLight
{
	vec3 vColor;
	vec3 vPosition;
	vec3 vDirection;
  
	// Used as boolean, whether spotlight is on
	int bOn;

	// If cone angle is set from program, we also provide its cosine,
	// so that it isn't always recalculated
	float fConeAngle, fConeCosine;
  
	// Only linear attenuation, you can code const and exp as well
	float fLinearAtt;
};

struct Material
{
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
	float shininess;
};

uniform SimpleDirectionalLight sunLight;
uniform SpotLight flashLight;
uniform Material material;

out vec4 FragColor;

vec4 getSpotLightColor(const SpotLight spotLight, vec3 vWorldPos, vec3 vNormal);
vec4 getSpotLightColor2(const SpotLight spotLight, vec3 vWorldPos, vec3 vNormal);
vec3 calcBumpedNormal();

void main(){

	// interpolated normal vector on current fragment
	// Calculate normal from normal map
	vec3 normal = calcBumpedNormal();

	// N dot L
	float lambertian = max(dot(-sunLight.vDirection, normal), 0.0);
	// specular coefficent
	float spec = 0.0;

	if (lambertian > 0.0){
		// TODO: eigentlich: normalize(cameraposition - vertexposition)
		//vec3 viewDir = normalize(vVertexPosition);
		vec3 viewDir = normalize(varVertexPosition);
		// Blinn Phong

		// Halfway-Vector (faster than reflection vector)
		vec3 halfDir = normalize(-sunLight.vDirection + viewDir);
		// N dot H
		float specularAngle = max(dot(halfDir, normal), 0.0);
		spec = pow(specularAngle, material.shininess*100);

	}

	// get texture color
	vec3 vTexColor = texture(colorTexture, vUVs).rgb;
	vec3 color;
	vec3 currentColor;
	vec2 texCoord = vUVs;
	
	vec3 normalBlur = normalize(varNormal);
	vec3 vDirec = varVertexPosition - flashLight.vPosition;
	vDirec = normalize(flashLight.vDirection); 
	if(dot(normalBlur, vDirec) >= -0.5 && (movement.x!=0 || movement.y!=0)){
		
		for(int i = 0; i < 10; i++)  
		{  
			texCoord += movement;
			//texCoord += vec2(0.0,0.0);
		  // Sample the color buffer along the velocity vector.  
		   currentColor += texture(colorTexture, texCoord).rgb; 
		 }  
		 // Average all of the samples to get the final blur color.  
		currentColor = currentColor / 10;
		
		vTexColor = currentColor; 
	}else{
		vTexColor = texture(colorTexture, vUVs).rgb;
	}
	
	vTexColor = clamp(vTexColor, 0.0, 1.0);

	// ignore alpha channel
	vec3 materialAmbient = material.ambientColor.xyz;
	vec3 materialDiffuse = material.diffuseColor.xyz;
	vec3 materialSpecular = material.specularColor.xyz;

	// calculate light parameters
	vec3 ambient = sunLight.fAmbientIntensity * materialAmbient * sunLight.vColor;
	vec3 diffuse = lambertian * materialDiffuse * sunLight.vColor;
	vec3 specular = spec * materialSpecular;

	// get intensity from lightmap
	vec3 vLightMapColor = texture(lightMap, vUVs).rgb;

	vec3 vSpotlightColor = vec3(getSpotLightColor2(flashLight, varVertexPosition, normal));

	// mix ambient, diffuse and specular part
	//gl_FragColor = vec4(vTexColor * (ambient + diffuse + vSpotlightColor) + specular, 1.0);

	if(transparent == 1){
		FragColor = vec4(vTexColor, alphaUni);
	} else {
		vec3 ambientColor = vec3(0.1, 0.1, 0.1);
		FragColor = clamp(vec4( ambientColor + vTexColor * (vSpotlightColor + vLightMapColor), 1.0), 0.0, 1.0);
	}

}


vec4 getSpotLightColor(const SpotLight spotLight, vec3 vWorldPos, vec3 vNormal)
{
	// If flashlight isn't turned on, return no color
	if(spotLight.bOn == 0)return vec4(0.0, 0.0, 0.0, 0.0);
	// Distance from fragment's position
	float fDistance = distance(vWorldPos, spotLight.vPosition);
  
	// Get direction vector to fragment
	vec3 vDir = vWorldPos - spotLight.vPosition;
	vDir = normalize(vDir);
  
	// Cosine between spotlight direction and directional vector to fragment
	float fCosine = dot(spotLight.vDirection, vDir);
  
	// Difference between max cosine and current cosine
	float fDif = 1.0-spotLight.fConeCosine;
  
	// This is how strong light is depending whether its nearer to the center of
	// cone or nearer to its borders (onway factor in article), clamp to 0.0 and 1.0
	float fFactor = clamp((fCosine-spotLight.fConeCosine)/fDif, 0.0, 1.0);
    
	// If we're inside cone, calculate color
	if(fCosine > spotLight.fConeCosine){
	//if (true){
		//return vec4(1.0,1.0,1.0,1.0);
		//return clamp(vec4(spotLight.vColor, 1.0)*fFactor/(fDistance*spotLight.fLinearAtt), 0.0, 1.0);
		//return clamp(vec4(spotLight.vColor, 1.0)*fFactor / (fDistance / 1 * spotLight.fLinearAtt), 0.0, 1.0);

		float diffuseFactor = dot(vNormal, -spotLight.vDirection);
		//if (diffuseFactor < 0){
			//diffuseFactor = dot(-vNormal, -spotLight.vDirection);
		//}

		vec4 diffuseColor  = vec4(0, 0, 0, 0);

		if (diffuseFactor > 0) {
			//diffuseColor = 0.5 * vec4(spotLight.vColor , 1.0) + 0.5 * vec4(spotLight.vColor, 1.0) * diffuseFactor;
			diffuseColor = vec4(spotLight.vColor, 1.0) * diffuseFactor;
		}

		float attenuation = fDistance / 1 * spotLight.fLinearAtt;

		return clamp(diffuseColor * fFactor / attenuation, 0.0, 1.0);
	}

	// No color otherwise
	return vec4(0.0, 0.0, 0.0, 1.0);
}

vec4 getSpotLightColor2(const SpotLight spotLight, vec3 vWorldPos, vec3 vNormal)
{

	// If flashlight isn't turned on, return no color
	if(spotLight.bOn == 0)return vec4(0.0, 0.0, 0.0, 0.0);
	// Distance from fragment's position
	float fDistance = distance(vWorldPos, spotLight.vPosition);
  
	// Get direction vector to fragment
	vec3 vDir = vWorldPos - spotLight.vPosition;
	vDir = normalize(vDir);
  
	// Cosine between spotlight direction and directional vector to fragment
	float fCosine = dot(spotLight.vDirection, vDir);
  
	// Difference between max cosine and current cosine
	float fDif = 1.0-spotLight.fConeCosine;
  
	// This is how strong light is depending whether its nearer to the center of
	// cone or nearer to its borders (onway factor in article), clamp to 0.0 and 1.0
	float fFactor = clamp((fCosine-spotLight.fConeCosine)/fDif, 0.0, 1.0);
    
	// If we're inside cone, calculate color
	if(fCosine > spotLight.fConeCosine){

		vec3 Normal = normalize(varNormal);
		vec3 Tangent = normalize(varTangent);
		Tangent = normalize(Tangent - dot(Tangent, Normal) * Normal);
		vec3 Bitangent = cross(Tangent, Normal);
		mat3 TBN = mat3(Tangent, Bitangent, Normal);

		vDir = spotLight.vPosition - vWorldPos;
		vDir = vDir*TBN;
		vDir = normalize(vDir);

		vec3 norm = texture2D(normalMap, vUVs).rgb * 2.0 - 1.0; 
		float nxDir = max(0.0, dot(norm, vDir)); 

		vec4 diffuseColor = vec4(spotLight.vColor, 1.0) * nxDir;
		float attenuation = fDistance / 1 * spotLight.fLinearAtt;
		//return diffuseColor / distance(vWorldPos, spotLight.vPosition);

		return clamp(diffuseColor * fFactor / attenuation, 0.0, 1.0);
		//return vec4(1.0, 1.0, 1.0, 1.0);
	}
	return vec4(0.0, 0.0, 0.0, 1.0);
}

vec3 calcBumpedNormal()
{
    vec3 Normal = normalize(varNormal);
    vec3 Tangent = normalize(varTangent);
    Tangent = normalize(Tangent - dot(Tangent, Normal) * Normal);
    vec3 Bitangent = cross(Tangent, Normal);
    vec3 BumpMapNormal = texture(normalMap, vUVs).xyz;
    BumpMapNormal = 2.0 * BumpMapNormal - vec3(1.0, 1.0, 1.0);
    vec3 NewNormal;
    mat3 TBN = mat3(Tangent, Bitangent, Normal);
    NewNormal = TBN * BumpMapNormal;
    NewNormal = normalize(NewNormal);
    return NewNormal;

	//return texture(normalMap, vUVs).xyz;
}